package kse.clientServer;

import org.jmock.Expectations;
import org.jmock.Mockery;
import org.jmock.integration.junit4.JUnit4Mockery;
import org.junit.Before;
import org.junit.Test;

/**
 *
 * @author mnovacek
 */
public class ServerTest {

    private Server server;

    @Before
    public void setUp() {
        server = new Server();
    }

    /**
     * Test of send method, of class Server.
     */
    @Test
    public void testSend() {
        Mockery context = new JUnit4Mockery();

        final IClient mockClient1 = context.mock(IClient.class, "client 1");
        final IClient mockClient2 = context.mock(IClient.class, "client 2");
        final IClient mockClient3 = context.mock(IClient.class, "client 3");
        server.register(mockClient1);
        server.register(mockClient2);
        server.register(mockClient3);

        final String message = "message";

        context.checking(new Expectations() {

            {
                oneOf(mockClient2).receive(mockClient1, message);
                never(mockClient1).receive(mockClient1, message);
                atLeast(1).of(mockClient2).isIgnored(mockClient1);
                will(returnValue(false));
                atLeast(1).of(mockClient3).isIgnored(mockClient1);
                will(returnValue(true));
                never(mockClient3).receive(mockClient1, message);
            }
        });

        server.send(mockClient1, message);

        context.assertIsSatisfied();
    }

    @Test
    public void testSendNullMessage() {
        Mockery context = new JUnit4Mockery();

        final IClient mockClient1 = context.mock(IClient.class, "client 1");
        final IClient mockClient2 = context.mock(IClient.class, "client 2");
        server.register(mockClient2);
        server.register(mockClient1);

        context.checking(new Expectations() {

            {
                atLeast(1).of(mockClient2).isIgnored(mockClient1);
                will(returnValue(false));
                oneOf(mockClient2).receive(mockClient1, null);
                will(throwException(new IllegalArgumentException("message can not be null")));
            }
        });

        server.send(mockClient1, null);

        context.assertIsSatisfied();
    }

    @Test
    public void testWishHappyBirthday() {
        Mockery context = new JUnit4Mockery();

        final IDate serverDate = context.mock(IDate.class, "server date");
        final IDate client1Birthday = context.mock(IDate.class, "client 1 birthday");

        final IClient mockClient1 = context.mock(IClient.class, "client 1");
        final IClient mockClient2 = context.mock(IClient.class, "client 2");


        server = new Server();
        server.setDate(serverDate);
        server.register(mockClient1);
        server.register(mockClient2);

        context.checking(new Expectations() {

            {
                // mocking client birthday
                atLeast(1).of(client1Birthday).getDay();
                will(returnValue(1));
                allowing(client1Birthday).getMonth();
                will(returnValue(1));
                never(client1Birthday).getYear();

                // mocking server date
                atLeast(2).of(serverDate).getDay();
                will(returnValue(1));
                atLeast(2).of(serverDate).getMonth();
                will(returnValue(2));
                never(serverDate).getYear();

                atLeast(1).of(mockClient1).getBirthday();
                will(returnValue(client1Birthday));
                atLeast(1).of(mockClient2).getBirthday();
                will(returnValue(serverDate));
                oneOf(mockClient2).receiveSystemMessage("Happy Birthday!!!");
                never(mockClient1).receiveSystemMessage(with(any(String.class)));
            }
        });

        server.wishHappyBirthday();

        context.assertIsSatisfied();
    }
}
