
import java.util.ArrayList;
import java.util.List;
import java.util.Observable;

public class Paragraph extends Observable {

    private List<String> lines;
    private Formatter formatter = new AlignLeft();
    private int maxLength = 0;

    /**
     * Set the currently used formatter.
     * @param newFormatter
     */
    public void setFormater(Formatter newFormatter) {
        formatter = newFormatter;
    }

    public Paragraph(Counter c) {
        lines = new ArrayList<String>();
        addObserver(c);
    }

    /**
     * Return the line at the i-th position.
     */
    public String getLine(int i) {
        return lines.get(i);
    }

    /**
     * Return the number of lines in the paragraph.
     */
    public int getCountLines() {
        return lines.size();
    }

    /**
     * Add a line to the paragraph.
     */
    public void addLine(String s) {
        lines.add(s);
        if (maxLength < s.length()) {
            maxLength = s.length();
        }
        setChanged();
        notifyObservers();
    }

    /**
     * Align the paragraph using the given formatter.
     */
    public List<String> alignedText() {
        return formatter.align(lines, maxLength);
    }
}
