package refactoringsolution;

import java.util.Arrays;

abstract class Matrix {

    protected int size;

    public int getSize() {
        return size;
    }

    @Override
    public boolean equals(Object obj) {
        if (!(obj instanceof Matrix)) {
            return false;
        }
        Matrix m = (Matrix) obj;

        if (getSize() != m.getSize()) {
            return false;
        }

        for (int i = 0; i < getSize(); i++) {
            for (int j = 0; j < getSize(); j++) {
                if (get(i, j) != m.get(i, j)) {
                    return false;
                }
            }
        }

        return true;
    }

    @Override
    public int hashCode() {
        int result = 17;

        for (int i = 0; i < getSize(); i++) {
            for (int j = 0; j < getSize(); j++) {
                result *= 31;
                result += get(i, j);
            }
        }

        return result;
    }

    @Override
    public String toString() {
        String result = "";
        for (int i = 0; i < getSize(); i++) {
            result += "[";
            for (int j = 0; j < getSize(); j++) {
                result += " " + get(i, j);
            }
            result += "]\n";
        }
        return result;
    }

    abstract void set(int i, int j, double newValue);

    abstract double get(int i, int j);

    public abstract Matrix add(Matrix m);

    protected abstract Matrix addDiagonal(DiagonalMatrix m);

    protected abstract FullMatrix addFull(FullMatrix m);
}

class FullMatrix extends Matrix {

    private double[][] elements;

    FullMatrix(int size) {
        this.size = size;
        elements = new double[size][size];
    }

    FullMatrix(FullMatrix m) {
        size = m.size;
        elements = new double[size][size];
        for (int i = 0; i < size; i++) {
            System.arraycopy(m.elements[i], 0, elements[i], 0, size);
        }
    }

    @Override
    public void set(int i, int j, double newValue) {
        elements[i][j] = newValue;
    }

    @Override
    public double get(int i, int j) {
        return elements[i][j];
    }

    @Override
    public Matrix add(Matrix m) {
        assert (getSize() == m.getSize());
        return m.addFull(this);
    }

    @Override
    protected FullMatrix addDiagonal(DiagonalMatrix m) {
        FullMatrix result = new FullMatrix(this);
        for (int i = 0; i < size; i++) {
            result.elements[i][i] += m.get(i, i);
        }
        return result;
    }

    @Override
    protected FullMatrix addFull(FullMatrix m) {
        FullMatrix result = new FullMatrix(this);
        for (int i = 0; i < size; i++) {
            for (int j = 0; j < size; j++) {
                result.elements[i][j] += m.elements[i][j];
            }
        }
        return result;
    }
}

class DiagonalMatrix extends Matrix {

    private double[] elements;

    DiagonalMatrix(int size) {
        this.size = size;
        elements = new double[size];
    }

    DiagonalMatrix(DiagonalMatrix m) {
        size = m.size;
        elements = new double[size];
        System.arraycopy(m.elements, 0, elements, 0, size);
    }

    @Override
    public void set(int i, int j, double newValue) {
        assert (i == j);
        elements[i] = newValue;
    }

    @Override
    public double get(int i, int j) {
        if (i == j) {
            return elements[i];
        } else {
            return 0.;
        }
    }

    @Override
    public Matrix add(Matrix m) {
        assert (getSize() == m.getSize());
        return m.addDiagonal(this);
    }

    @Override
    protected DiagonalMatrix addDiagonal(DiagonalMatrix m) {
        DiagonalMatrix result = new DiagonalMatrix(this);
        for (int i = 0; i < size; i++) {
            result.elements[i] += m.elements[i];
        }
        return result;
    }

    @Override
    protected FullMatrix addFull(FullMatrix m) {
        return m.addDiagonal(this);
    }
}

public class ArrayOfMatrix {

    private Matrix[] matrices;
    private int length;

    ArrayOfMatrix(int size) {
        this.length = size;
        matrices = new Matrix[size];
    }

    @Override
    public boolean equals(Object obj) {
        if (!(obj instanceof ArrayOfMatrix)) {
            return false;
        }
        ArrayOfMatrix ar = (ArrayOfMatrix) obj;

        if (length != ar.length) {
            return false;
        }

        for (int i = 0; i < length; i++) {
            if (matrices[i] != null && ar.matrices[i] == null) {
                return false;
            } else if (matrices[i] == null && ar.matrices[i] != null) {
                return false;
            } else if (matrices[i] != null && ar.matrices[i] != null && !matrices[i].equals(ar.matrices[i])) {
                return false;
            }
        }
        return true;
    }

    @Override
    public int hashCode() {
        return Arrays.hashCode(matrices);
    }

    public void set(int i, Matrix newVal) {
        matrices[i] = newVal;
    }

    public ArrayOfMatrix add(ArrayOfMatrix ar) {
        assert (length == ar.length);

        ArrayOfMatrix result = new ArrayOfMatrix(length);

        for (int ind = 0; ind < length; ind++) {
            result.matrices[ind] = matrices[ind].add(ar.matrices[ind]);
        }
        return result;
    }
}
