package refactoringsolution;

import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author Arsenii
 */
public class ArrayOfMatrixTest {

    private final int n = 2;
    private final int m = 4;
    private DiagonalMatrix dm1, dm2, dm3;
    private FullMatrix fm1, fm2, fm3, fm4, fm5;
    private ArrayOfMatrix am1, am2, am3, amNotNull, amNull;

    @Before
    public void setUp() {

        fm1 = new FullMatrix(n);
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                fm1.set(i, j, i + j);
            }
        }

        fm2 = new FullMatrix(n);
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                fm2.set(i, j, 2 * (i + j));
            }
        }

        fm3 = new FullMatrix(n);
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                fm3.set(i, j, 3 * (i + j));
            }
        }

        fm4 = new FullMatrix(n);
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                fm4.set(i, j, i == j ? 5 * i : 2 * (i + j));
            }
        }

        fm5 = new FullMatrix(n);
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                fm5.set(i, j, i == j ? 4 * i : i + j);
            }
        }

        dm1 = new DiagonalMatrix(n);
        for (int i = 0; i < n; i++) {
            dm1.set(i, i, i);
        }

        dm2 = new DiagonalMatrix(n);
        for (int i = 0; i < n; i++) {
            dm2.set(i, i, 2 * i);
        }

        dm3 = new DiagonalMatrix(n);
        for (int i = 0; i < n; i++) {
            dm3.set(i, i, 3 * i);
        }


        am1 = new ArrayOfMatrix(m);
        am1.set(0, new FullMatrix(fm1));
        am1.set(1, new DiagonalMatrix(dm1));
        am1.set(2, new FullMatrix(fm1));
        am1.set(3, new DiagonalMatrix(dm1));

        am2 = new ArrayOfMatrix(m);
        am2.set(0, new FullMatrix(fm2));
        am2.set(1, new FullMatrix(fm2));
        am2.set(2, new DiagonalMatrix(dm2));
        am2.set(3, new DiagonalMatrix(dm2));

        am3 = new ArrayOfMatrix(m);
        am3.set(0, new FullMatrix(fm3));
        am3.set(1, new FullMatrix(fm4));
        am3.set(2, new FullMatrix(fm5));
        am3.set(3, new DiagonalMatrix(dm3));

        amNotNull = new ArrayOfMatrix(1);
        amNotNull.set(0, new FullMatrix(1));

        amNull = new ArrayOfMatrix(1);
        amNull.set(0, null);
    }

    /**
     * Test of hashCode method, of class ArrayOfMatrix.
     */
    @Test
    public void testHashCode() {
        final int size = 2;
        ArrayOfMatrix am = new ArrayOfMatrix(size);
        ArrayOfMatrix amCopy = new ArrayOfMatrix(size);

        am.set(0, new FullMatrix(fm1));
        am.set(1, new DiagonalMatrix(dm1));

        amCopy.set(0, new FullMatrix(fm1));
        amCopy.set(1, new DiagonalMatrix(dm1));

        assertEquals("ArrayOfMatrix.Set error", am.hashCode(), amCopy.hashCode());
    }

    /**
     * Test of set and equals methods, of class ArrayOfMatrix.
     */
    @Test
    public void testSet() {

        final int size = 2;
        ArrayOfMatrix am = new ArrayOfMatrix(size);
        ArrayOfMatrix amCopy = new ArrayOfMatrix(size);

        am.set(0, new FullMatrix(fm1));
        am.set(1, new DiagonalMatrix(dm1));

        amCopy.set(0, new FullMatrix(fm1));
        amCopy.set(1, new DiagonalMatrix(dm1));

        assertEquals("ArrayOfMatrix.Set error", am, amCopy);
    }

    /**
     * Test of add and equals methods, of class ArrayOfMatrix.
     */
    @Test
    public void testAddEquals() {
        assertEquals("ArrayOfMatrix.Add error", am1.add(am2), am3);
    }

    @Test(expected = AssertionError.class)
    public void testAddFails() {
        new ArrayOfMatrix(2).add(new ArrayOfMatrix(1));
    }

    @Test
    public void testNotEqualsNullAndNotNull() {
        assertFalse("ArrayOfMatrix.equals error", amNull.equals(amNotNull));
    }

    @Test
    public void testNotEqualsNotNullAndNull() {
        assertFalse("ArrayOfMatrix.equals error", amNotNull.equals(amNull));
    }

    @Test
    public void testNotEqualsDifferentContent() {
        assertFalse("ArrayOfMatrix.equals error", am1.equals(am2));
    }

    @Test
    public void testNotEqualsDifferentSize() {
        assertFalse("ArrayOfMatrix.equals error", amNotNull.equals(am1));
    }

    @Test
    public void testNotEqualArrayOfMatixToString() {
        assertFalse("ArrayOfMatrix.equals error", am1.equals("Matrix"));
    }
}
