package refactoringtask;

abstract class Matrix {

    protected int size;

    public int getSize() {
        return size;
    }

    @Override
    public boolean equals(Object obj) {
        if (!(obj instanceof Matrix))
            return false;
        Matrix m = (Matrix) obj;

        if (getSize() != m.getSize())
            return false;

        for (int i = 0; i < getSize(); i++)
            for (int j = 0; j < getSize(); j++)
                if (get(i, j) != m.get(i, j)) 
                    return false;

        return true;
    }

    abstract void set(int i, int j, double newValue);

    abstract double get(int i, int j);
}

class FullMatrix extends Matrix {

    private double[][] elements;

    FullMatrix(int size) {
        this.size = size;
        elements = new double[size][size];
    }

    FullMatrix(FullMatrix m) {
        size = m.size;
        elements = new double[size][size];
        for (int i = 0; i < size; i++)
            for (int j = 0; j < size; j++)
                elements[i][j] = m.elements[i][j];
    }

    public void set(int i, int j, double newValue) {
        elements[i][j] = newValue;
    }

    public double get(int i, int j) {
        return elements[i][j];
    }
}

class DiagonalMatrix extends Matrix {

    private double[] elements;

    DiagonalMatrix(int size) {
        this.size = size;
        elements = new double[size];
    }

    DiagonalMatrix(DiagonalMatrix m) {
        size = m.size;
        elements = new double[size];
        for (int i = 0; i < size; i++)
            elements[i] = m.elements[i];
    }

    public void set(int i, int j, double newValue) {
        assert (i == j);
        elements[i] = newValue;
    }

    public double get(int i, int j) {
        if (i == j)
            return elements[i];
        else
            return 0.;
    }
}

public class ArrayOfMatrix {

    private Matrix[] matrices;
    private int length;

    ArrayOfMatrix(int size) {
        this.length = size;
        matrices = new Matrix[size];
    }

    @Override
    public boolean equals(Object obj) {
        if (!(obj instanceof ArrayOfMatrix))
            return false;

        ArrayOfMatrix ar = (ArrayOfMatrix) obj;

        if (length != ar.length)
            return false;

        for (int i = 0; i < length; i++)
            if (matrices[i] != null && ar.matrices[i] == null)
                return false;
            else if (matrices[i] == null && ar.matrices[i] != null)
                return false;
            else if (matrices[i] != null && ar.matrices[i] != null && !matrices[i].equals(ar.matrices[i]))
                return false;
        return true;
    }

    public void set(int i, Matrix newVal) {
        matrices[i] = newVal;
    }

    public ArrayOfMatrix add(ArrayOfMatrix ar) {
        assert (length == ar.length);

        ArrayOfMatrix result = new ArrayOfMatrix(length);

        for (int ind = 0; ind < length; ind++) {
            Matrix resultTemp;
            if (matrices[ind] instanceof FullMatrix) {
                FullMatrix m1 = (FullMatrix) matrices[ind];
                if (ar.matrices[ind] instanceof FullMatrix) {
                    FullMatrix m2 = (FullMatrix) ar.matrices[ind];
                    assert (m1.getSize() == m2.getSize());
                    FullMatrix temp = new FullMatrix(m1);
                    for (int i = 0; i < m1.getSize(); i++)
                        for (int j = 0; j < m1.getSize(); j++)
                            temp.set(i, j, temp.get(i, j) + m2.get(i, j));
                    resultTemp = temp;
                } else {
                    DiagonalMatrix m2 = (DiagonalMatrix) ar.matrices[ind];
                    assert (m1.getSize() == m2.getSize());
                    FullMatrix temp = new FullMatrix(m1);
                    for (int i = 0; i < m1.getSize(); i++)
                        temp.set(i, i, temp.get(i, i) + m2.get(i, i));
                    resultTemp = temp;
                }
            } else {
                DiagonalMatrix m1 = (DiagonalMatrix) matrices[ind];
                if (ar.matrices[ind] instanceof FullMatrix) {
                    FullMatrix m2 = (FullMatrix) ar.matrices[ind];
                    assert (m1.getSize() == m2.getSize());
                    FullMatrix temp = new FullMatrix(m2);
                    for (int i = 0; i < m1.getSize(); i++)
                        temp.set(i, i, temp.get(i, i) + m1.get(i, i));
                    resultTemp = temp;
                } else {
                    DiagonalMatrix m2 = (DiagonalMatrix) ar.matrices[ind];
                    assert (m1.getSize() == m2.getSize());
                    DiagonalMatrix temp = new DiagonalMatrix(m1);
                    for (int i = 0; i < m1.getSize(); i++)
                        temp.set(i, i, temp.get(i, i) + m2.get(i, i));
                    resultTemp = temp;
                }
            }
            result.matrices[ind] = resultTemp;
        }

        return result;
    }
}
