/*
 * SlideShow.java
 *
 * Created on 23.03.2010, 14:04:52
 */

import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FilenameFilter;
import java.util.Deque;
import java.util.concurrent.LinkedBlockingDeque;
import javax.swing.ImageIcon;
import javax.swing.Timer;

public class SlideShow extends javax.swing.JFrame {

    private boolean isRunning = false;
    // speed in seconds
    private volatile int speed;
    private volatile Timer timer;
    private Deque<ImageIcon> images = new LinkedBlockingDeque<ImageIcon>();
    private ImageIcon currentImage;

    private void loadImages(File directory) {
        File[] files = directory.listFiles(new FilenameFilter() {

            public boolean accept(File dir, String name) {
                return name.endsWith(".png");
            }
        });
        for (File f : files) {
            System.out.println("Found image: " + f.getAbsoluteFile());
            images.add(new ImageIcon(f.getAbsolutePath().toString()));
        }
    }

    private void startOrPause() {
        if (isRunning) {
            toggleButton.setText("Start");
            timer.stop();
        } else {
            ImageIcon next = nextImage();
            setCurrentImage(next);
            toggleButton.setText("Pause");
            timer.restart();
        }
        isRunning = !isRunning;
    }

    private void setCurrentImage(ImageIcon next) {
        imageLabel.setIcon(next);
        resizeCurrentImage();
    }

    private void resizeCurrentImage() {
        if (currentImage != null) {
            Image resizedImage = currentImage.getImage().getScaledInstance(
                    -1, imageLabel.getHeight(), java.awt.Image.SCALE_SMOOTH);
            imageLabel.setIcon(new ImageIcon(resizedImage));
        }
    }

    private ImageIcon nextImage() {
        if (!images.isEmpty()) {
            currentImage = images.pop();
            images.addLast(currentImage);
            System.out.println("Current image: " + currentImage.toString());
            return currentImage;
        }
        return null;
    }

    private void changeSpeed(int speed) {
        this.speed = speed;

        int delay = speed * 1000;

        if (timer == null) {
            timer = new Timer(delay, new ActionListener() {

                public void actionPerformed(ActionEvent e) {
                    setCurrentImage(nextImage());
                }
            });
        } else {
            // timer.stop();
            timer.setDelay(delay);
            timer.setInitialDelay(delay);
            timer.restart();
            // timer.start();
        }

    }

    /** Creates new form SlideShow */
    public SlideShow() {
        initComponents();
        changeSpeed(speedSlider.getValue());
        loadImages(new File("Images"));
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        imageLabel = new javax.swing.JLabel();
        toggleButton = new javax.swing.JToggleButton();
        speedSlider = new javax.swing.JSlider();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setMinimumSize(new java.awt.Dimension(310, 334));
        addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentResized(java.awt.event.ComponentEvent evt) {
                formComponentResized(evt);
            }
        });

        imageLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        imageLabel.setMinimumSize(new java.awt.Dimension(200, 200));
        imageLabel.setName("Current Image"); // NOI18N
        imageLabel.setPreferredSize(null);

        toggleButton.setText("Start");
        toggleButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                toggleButtonActionPerformed(evt);
            }
        });

        speedSlider.setMajorTickSpacing(1);
        speedSlider.setMaximum(9);
        speedSlider.setMinimum(1);
        speedSlider.setPaintTicks(true);
        speedSlider.setSnapToTicks(true);
        speedSlider.setValue(5);
        speedSlider.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                speedSliderStateChanged(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addGap(57, 57, 57)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.CENTER)
                    .addComponent(imageLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 200, Short.MAX_VALUE)
                    .addComponent(speedSlider, javax.swing.GroupLayout.PREFERRED_SIZE, 100, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(toggleButton))
                .addGap(53, 53, 53))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(imageLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 206, Short.MAX_VALUE)
                .addGap(18, 18, 18)
                .addComponent(toggleButton)
                .addGap(18, 18, 18)
                .addComponent(speedSlider, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(27, 27, 27))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void toggleButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_toggleButtonActionPerformed
        startOrPause();
    }//GEN-LAST:event_toggleButtonActionPerformed

    private void speedSliderStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_speedSliderStateChanged
        changeSpeed(speedSlider.getValue());
    }//GEN-LAST:event_speedSliderStateChanged

    private void formComponentResized(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_formComponentResized
        resizeCurrentImage();
    }//GEN-LAST:event_formComponentResized

    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {

            public void run() {
                new SlideShow().setVisible(true);
            }
        });
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel imageLabel;
    private javax.swing.JSlider speedSlider;
    private javax.swing.JToggleButton toggleButton;
    // End of variables declaration//GEN-END:variables
}
